#ifndef CCLASSFACTORY_H
#define CCLASSFACTORY_H

class DECLSPEC_NOVTABLE CClassFactory : public IClassFactory
{
public:
    // IUnknown
    IFACEMETHOD(QueryInterface) (REFIID riid, void** ppvObject);
    IFACEMETHOD_(ULONG, AddRef) ();
    IFACEMETHOD_(ULONG, Release)();

    // IClassFactory
    IFACEMETHOD(LockServer)(BOOL fLock);

    CClassFactory()  throw();
    virtual ~CClassFactory() throw();

private:
    volatile ULONG m_lRefCount;
#ifdef _WIN64
    BYTE bPadding[4]; // C4820
#endif
};

template<typename T>
class CClassFactoryT : public CClassFactory
{
public:
    // IClassFactory
    IFACEMETHOD(CreateInstance)(IUnknown* pUnkOuter, REFIID riid, void** ppvObject)
    {
        HRESULT hr = S_OK;
        T*      pcls;

        if(IsBadWritePtr(ppvObject, sizeof(*ppvObject)))
            return(E_POINTER);

        *ppvObject = nullptr;

        if(pUnkOuter)
            return(CLASS_E_NOAGGREGATION);

        if(nullptr == (pcls = new (std::nothrow) T(&hr)))
            return(E_OUTOFMEMORY);

        if(SUCCEEDED(hr))
            hr = pcls->QueryInterface(riid, ppvObject);

        pcls->Release();
        return(hr);
    }
};

#endif
